/*******************************************************************************
				Neoinput
				========

		Copyright (C) 2010 Paul Kocialkowski

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "client.h"
#include "config.h"
#include "input_preset.h"
#include "input_write.h"


void print_version(void)
{
	printf("%s_client %s, %s\n", SOFTWARE, VERSION, DESCRIPTION);
	printf("%s\n", COPYRIGHT);
}

void print_help(void)
{
	print_version();
	printf("\nUsage: %s_client [OPTION]... [PRESET]\n", SOFTWARE);
	printf("\nOptions:\n");
	printf("  --help        Shows this help message.\n");
	printf("  --version     Shows the program version.\n");
	printf("  -a [ADDRESS]  Specify the IP address to connect on.\n");
	printf("  -p [PORT]     Specify the port to connect on.\n");
	printf("  -d [FILE]     Specify the device to inject data on. Default is uinput.\n");
	printf("\nPresets:\n");
	printf("  Default presets are:\n");
	printf("touch           Use the touchscreen (and the buttons)\n");
	printf("touch-rotate    Use the 90deg rotated touchscreen (and the buttons)\n");
	printf("accel-mania     Use the accelerometers with the ManiaDrive game\n");
	printf("accel-plee      Use the accelerometers with the Plee The Bear game\n");
}

int main(int argc, char **argv)
{
	int i, j;

	int server_port=0;
	char *server_ip=NULL;

	char append=0;

	/* Check args */

	for(i=1 ; i < argc ; i++)
	{
		if(argv[i][0] == '-')
		{
			if(argv[i][1] == '-')
			{
				if(strcmp(argv[i], "--help") == 0)
				{
					print_help();
					exit(0);
				}
				else if(strcmp(argv[i], "--version") == 0)
				{
					print_version();
					exit(0);
				}
				else
				{
					printf("Error: Unknown argument '%s'.\n", argv[i]);
					exit(1);
				}
			}
			else
			{
				for(j=1 ; j < strlen(argv[i]) ; j++)
				{
					if(append)
					{
						printf("Error: missing value for argument '%c'.\n", append);
						exit(1);
					}
					
					switch(argv[i][j])
					{
						case 'a':
							append='a';
						break;

						case 'd':
							append='d';
						break;

						case 'p':
							append='p';
						break;

						default:
							printf("Error: Unknown argument '%c'.\n", argv[i][j]);
						break;
					}
				}
			}
		}
		else
		{
			if(append)
			{
				if(append == 'd' && input_device == NULL)
					input_device=argv[i];
				else if(append == 'p' && server_port == 0)
					server_port=atoi(argv[i]);
				else if(append == 'a' && server_ip == NULL)
					server_ip=argv[i];
				else
				{
					printf("Error: Argument '%c' redifinition.\n", append);
					exit(1);
				}

				append=0;
			}
			else if(input_preset == NULL)
			{
				input_preset=argv[i];
			}
			else
			{
				printf("Error: Unknown argument '%s'.\n", argv[i]);
				exit(1);
			}
		}
	}

	if(append)
	{
		printf("Error: missing value for argument '%c'.\n", append);
		exit(1);
	}

	if(server_ip == NULL)
		server_ip=DEFAULT_IP;
	printf("Connect on IP: %s\n", server_ip);

	if(server_port == 0)
		server_port=DEFAULT_PORT;
	printf("Connect on port: %d\n", server_port);

	if(input_preset == NULL)
		input_preset="touch";
	printf("Preset: %s\n", input_preset);

	if(input_device == NULL)
	{
		input_device=NULL;
		printf("Input device: uinput\n");
	}
	else
		printf("Input device: %s\n", input_device);


	/* Connect the server */
	server_connect(server_ip, server_port);

	/* Read the preset */
	input_preset_read(input_preset);

	/* Send the input type */
	server_send_device();

	/* Open the input device */
	input_write_open(input_device);

	/* Get the input events */
	input_event_get();

	/* Close the input device */
	input_write_close();

	/* Disconnect the server */
	server_disconnect();
}
